import json
from os import path as ospath

script_path = ospath.realpath(__file__)
script_dir_path = ospath.dirname(script_path)


def load_json(f):
    with open(f, 'r') as of:
        return json.load(of)


def get_data_file_path(file):
    return ospath.join(script_dir_path, 'datasets', file)


def get_lang_codes() -> dict:
    return load_json(get_data_file_path('lang.json'))





from ..utils import get_numbers_from_string


def sort_numeric(items, preformat=None):
    """Sorted lists based on the number contained in the items"""
    if preformat is None:
        pre_f = get_numbers_from_string
    else:
        pre_f = lambda x: get_numbers_from_string(preformat(x))
    return sorted(items, key=pre_f)




from rich.console import Console
from rich.traceback import install
from rich.terminal_theme import TerminalTheme
from .decorators import callCounter
import sys

console = Console(record=True, highlight=False, force_terminal=True, soft_wrap=True)

debug_active = True

theme = TerminalTheme(
    background=(30, 30, 30),
    foreground=(255, 255, 255),
    normal=[
        (0, 0, 0),
        (128, 0, 0),
        (0, 128, 0),
        (128, 128, 0),
        (0, 0, 128),
        (128, 0, 128),
        (0, 128, 128),
        (192, 192, 192),
    ],
    bright=[
        (128, 128, 128),
        (255, 0, 0),
        (0, 255, 0),
        (255, 255, 0),
        (0, 0, 255),
        (255, 0, 255),
        (0, 255, 255),
        (255, 255, 255),
    ]
)

console_input = console.input


def install_traceback():
    install(console=console, show_locals=True)  # rich traceback


@callCounter
def print_exception():
    console.print_exception(show_locals=True)


def print(*mess, style="", highlight=True):
    console.print(' '.join(mess), style=style, end="", highlight=highlight)


def println(*mess, style="", highlight=True):
    console.print(' '.join(mess), style=style, highlight=highlight)


def debug(*mess, style="", highlight=True):
    if debug_active:
        console.print(' '.join(mess), style=style, highlight=highlight)


def message(prefix, *mess, style=""):
    console.print(f"[{prefix}] " + ' '.join(mess), style=style)


@callCounter
def error(*mess, wait=False):
    console.print('[Error]', ' '.join(mess), style="red")
    if wait:
        console.input("[ENTER]")
        clear_lines(1)


@callCounter
def warning(*mess, wait=False):
    end = '\t[enter]' if wait else '\n'
    console.print("[Warning]", ' '.join(mess), style="yellow", end=end)
    if wait:
        console.input("[ENTER]")
        clear_lines(1)


def write_ending_log():
    console.print(f"[yellow]{warning.getFullCount()} - Warnings\n"
                  f"[red]{error.getFullCount()} - Error\n"
                  f"[red bold]{print_exception.getFullCount()} - ! Critical-Error !")


def clear_lines(n: int):
    for i in range(0, n):
        sys.stdout.write('\x1b[1A' + '\x1b[2K')


def save_html(path="log.html"):
    console.save_html(path, theme=theme)



import regex as re
import validators
from fnmatch import fnmatchcase as fnmatch
from ..regs.patterns import pattern_number


def is_image(file):
    """
    check if file has a ending like a image
    """  # Todo with endswith and list for checking
    _reg = re.compile(".(png|jpeg|jpg|gif|tif|bmp|swf|svg|webp)$", flags=re.IGNORECASE)

    if _reg.search(file):
        return True
    else:
        return False


def is_url(url):
    return validators.url(url)


def is_iterable(x):
    try:
        iter(x)
    except TypeError:
        return False
    else:
        return True


def is_float(value):
    try:
        float(value)
        return True
    except (ValueError, TypeError):
        return False


def fn_match_value(value: str, m: str | list):
    if isinstance(m, str):
        return fnmatch(value, m)
    if is_iterable(m):
        for n in m:
            if fn_match_value(value, n):
                return True
        return False


def remove_double(v: list):
    return list(dict.fromkeys(v))


def convert_to_int(value: str | None, default=None):
    if value is None:
        return default

    if value.isdigit():
        return int(value)

    value = re.sub('^([^0-9]*)([0-9]+)([^0-9]*)$', r'\2', value)
    if value.isdigit():
        return int(value)

    return default


def convert_to_float(value: str | None, default=None):
    if value is None:
        return default

    try:
        r = float(value)
        return r
    except ValueError:
        pass

    value = re.sub('^([^0-9]*)([0-9.]+)([^0-9]*)$', r'\2', value)

    try:
        r = float(value)
        return r
    except ValueError:
        pass

    return default


def get_numbers_from_string(var: str):
    numbers = pattern_number.findall(var)
    numbers = [int(numeric_string) for numeric_string in numbers]
    return numbers



from json import JSONEncoder
import json


class JEncoder(JSONEncoder):

    def default(self, obj):
        if hasattr(obj, 'to_json'):
            return obj.to_json()
        if hasattr(obj, 'to_dict'):
            return obj.to_dict()
        if hasattr(obj, '__list__'):
            return obj.__list__
        if hasattr(obj, '__dict__'):
            return obj.__dict__
        return JSONEncoder.default(self, obj)

    @staticmethod
    def to_json_string(value):
        return json.dumps(value, cls=JEncoder, indent=4)





# pip install regex
import regex as re


def replace_ignore_case(s: str, replace: str, val: str):
    return re.sub(re.escape(replace), val, s, flags=re.IGNORECASE)


class regReplace:
    def __init__(self, r, ingnoreCase=False):
        if ingnoreCase:
            self.r = re.compile(r, flags=re.IGNORECASE)
        else:
            self.r = re.compile(r)

    def __call__(self, value, repl):
        return self.r.sub(repl, value)


class regGroup:
    def __init__(self, r, flags=0):
        self.r = re.compile(r, flags=flags)

    def __call__(self, value, multi=False):
        if multi:
            return [m.groupdict() for m in self.r.finditer(value)]
        else:
            _m = self.r.search(value)
            if _m is None:
                return None
            else:
                return _m.groupdict()


class multiRegMatching:
    def __init__(self, *regs, flags=0):
        self.regs = [re.compile(reg, flags=flags) for reg in regs]

    def __call__(self, value):
        for r in self.regs:
            _m = r.search(value)
            if _m is None:
                continue
            else:
                return _m.groupdict()
        return None
